(function () {
    'use strict';

    angular
        .module('auth.module')
        .service('AuthServices', AuthServices);

    AuthServices.$inject = ['$http', 'ENV', '$rootScope', '$q'];

    /* @ngInject */
    function AuthServices($http, ENV, $rootScope, $q) {

        return {
            login: function (email, password, provider_id, provider_token) {

                return $http({
                    method: "post",
                    url: ENV.apiUrl + 'login',
                    headers: { 'Content-Type': 'application/json' },
                    data: {
                        email: email,
                        password: password,
                        provider_id: provider_id,
                        provider_token: provider_token
                    }
                }).then(function (response) {

                    return {
                        state: 'OK',
                        user: response.data
                    };

                }, function (response) {

                    return {
                        state: 'KO',
                        msg: response.statusText
                    };

                });
            },
            getLoggedUser: function () {
                return localStorage.getItem('user') ?
                    JSON.parse(localStorage.getItem('user')) : undefined;
            },

            logout: function () {
              var p = $q.defer();
              try {
                localStorage.removeItem('user');
                $rootScope.$broadcast('mt-user-logged');
                p.resolve(true);
              } catch (e) {
                p.resolve(false);
              }
              return p.promise;
            },
            register: function (email, password, provider_id, provider_uid, provider_token, data) {

              var data = {};
              if(provider_id!='1' ){
                data = {provider_id : provider_id,
                provider_uid: provider_uid,
                provider_token: provider_token};

              }else{
                data = {
                    credential: {
                        email: email,
                        password:password,
                        provider_id : provider_id,
                        provider_uid: provider_uid,
                        provider_token: provider_token
                    },
                    settings : {
                        "newsletter" : true
                    },
                };
              }


                return $http({
                    method: "post",
                    url: ENV.apiUrl + 'register'+(provider_id!='1' ? '/social' : ''),
                    headers: { 'Content-Type': 'application/json' },
                    data: data
                }).then(function (response) {
                    // alert(JSON.stringify(response));
                    return {
                        state: 'OK',
                        user: response.data
                    };

                }, function (response) {

                    var msg = '11000'==response.data.code ? 'E-mail esistente' : response.statusText;
                    return {
                        state: 'KO',
                        msg: msg
                    };

                });
            },
            recoveryPassword: function (email) {

                return $http({
                    method: "post",
                    url: ENV.apiUrl + 'recover',
                    headers: { 'Content-Type': 'application/json' },
                    data: {
                        credential: {
                            email: email
                        }
                    }
                }).then(function (response) {

                    return {
                        state: 'OK'
                    };

                }, function (response) {

                    return {
                        state: 'KO',
                        msg: response.statusText
                    };

                });
            }
        };
    }
})();
