(function () {
    'use strict';

    angular
      .module('events.module')
      .value('LocationData',{coords:{}, placeName:'',geocoded:{},isConsistent:false})
      .service('EventsServices', ['$rootScope', '$http', '$timeout','ENV', 'AuthServices','LocationData','$cordovaGeolocation','$log','$q','$ionicPlatform', 'uiGmapGoogleMapApi','$base64','$interpolate',
          function ($rootScope, $http, $timeout, ENV, AuthServices, LocationData, $cordovaGeolocation, $log, $q,$ionicPlatform, uiGmapGoogleMapApi, $base64, $interpolate) {

              return {
                validateEMail: function(email) {
                    var re = /^(([^<>()\[\]\\.,;:\s@"]+(\.[^<>()\[\]\\.,;:\s@"]+)*)|(".+"))@((\[[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}])|(([a-zA-Z\-0-9]+\.)+[a-zA-Z]{2,}))$/;
                    return re.test(email);
                },
                createFilters: function(){
                  var _this=this;

                  var today = new Date(); today.setHours(0,0,0,0);
                  var weekStart = _this.getMonday(new Date());
                  var weekEnd = new Date(weekStart); weekEnd.setDate(weekStart.getDate()+7);
                  today = new Date(); today.setHours(0,0,0,0);
                  var monthStart = new Date(today.getFullYear(), today.getMonth(), 1);
                  var monthEnd = new Date(today.getFullYear(), today.getMonth() + 1, 0);

                  return [
                    {code:'P',lbl: $interpolate("{{ 'event_list_evidence'  | translate }}")($rootScope),//'In evidenza',
                      filter:{ page: 1, count: 10 }
                    },
                    {code:'W',lbl:$interpolate("{{ 'event_list_thisweek'  | translate }}")($rootScope),//'Questa settimana',
                      filter:{
                          start_date: _this.format_iso8601(weekStart,0 ),
                          end_date: _this.format_iso8601(weekEnd,0 ),
                          page: 1,
                          count: 10
                      }
                    },
                    {code:'M',lbl:$interpolate("{{ 'event_list_thismonth'  | translate }}")($rootScope),//'Questo mese',
                      filter:{
                          start_date: _this.format_iso8601(monthStart,0),
                          end_date: _this.format_iso8601(monthEnd,0),
                          page: 1,
                          count: 10
                      }
                    },
                  ];
                },

                to2digits: function (n) {
                  return n < 10 ? '0' + n : n.toString();
                },

                format_iso8601: function (date, offset) {
                    var d = new Date(date.getTime() - offset * 60000);
                    var s = d.getFullYear() + '-' +
                        this.to2digits(d.getMonth() + 1) + '-' +
                        this.to2digits(d.getDate()) +
                        'T00:00:00+00:00';
                        // 'T22:00:00+00:00';

                    return s;
                },

                decodeAddress:function(address){
                  var res = '';
                  var cntry = '';
                  for (var i = 0; i < address.address_components.length; i++) {

                    var c = address.address_components[i];
                    // //console.log(("check geocode:"+angular.toJson(c));
                    if(c.types.length>0){
                      if(res==''   && c.types[0]=='locality'){
                        res =  c.long_name;
                        break;
                      }
                      // if(cntry=='' && c.types[0]=='country'){
                      //   cntry =  c.short_name;
                      // }
                      // if(res!='' && cntry!='') break;
                    }

                  }
                  return  (res!='' ) ? (res ) : results[0].formatted_address; ;
                  //return  (res!='' || cntry!='') ? (res +', '+cntry) : results[0].formatted_address; ;
                },

                getCurrentPlaceName:function ( coords, updateData) {
                    //$log.info('getCurrentPlaceName');
                    var _this=this;
                    uiGmapGoogleMapApi.then(function(maps) {
                      try {
                        if(google){
                          var geocoder = new google.maps.Geocoder();
                          var latlng = new google.maps.LatLng( coords.latitude, coords.longitude);

                          geocoder.geocode({'latLng': latlng}, function(results, status)
                            {
                                //$log.info('getCurrentPlaceName then');
                                if (status == google.maps.GeocoderStatus.OK)   {
                                  var retval = '';
                                    if (results[0]) {

                                      retval = _this.decodeAddress(results[0] );

                                    } else {
                                      retval =  '(non trovato)';
                                    }

                                } else {
                                    retval = '(non trovato)';
                                }
                                if(updateData) LocationData.placeName = retval;
                                $rootScope.$broadcast('mt-place-changed', retval);
                            });
                        }
                      } catch (e) {
                        retval = '(non trovato)';
                        $log.error('no current placename:'+e);
                      }
                    });
                },

                getCurrentPlaceNamePromise:function ( coords, updateData) {
                    var p = $q.defer();
                    //$log.info('getCurrentPlaceName');
                    var _this=this;
                    uiGmapGoogleMapApi.then(function(maps) {
                      try {
                        if(google){
                          var geocoder = new google.maps.Geocoder();
                          var latlng = new google.maps.LatLng( coords.latitude, coords.longitude);

                          geocoder.geocode({'latLng': latlng}, function(results, status)
                            {
                                //$log.info('getCurrentPlaceName then');
                                if (status == google.maps.GeocoderStatus.OK)   {
                                  var retval = '';
                                    if (results[0]) {

                                      retval = _this.decodeAddress(results[0] );
                                      p.resolve(retval);
                                    } else {
                                      retval =  '(non trovato)';
                                      p.reject(false);
                                    }

                                } else {
                                    retval = '(non trovato)';
                                    p.reject(false);
                                }
                                if(updateData) LocationData.placeName = retval;
                                $rootScope.$broadcast('mt-place-changed', retval);
                            });
                        }
                      } catch (e) {
                        retval = '(non trovato)';
                        $log.error('no current placename:'+e);
                      }
                    });
                    return p.promise;
                },

                getCurrentPosition:function ( f ) {
                  var _this=this;
                    //$log.info('getCurrentPosition');
                    var posOptions = {timeout: 30000, enableHighAccuracy: false};

                    $ionicPlatform.ready(function () {
                      // will execute when device is ready, or immediately if the device is already ready.

                      $cordovaGeolocation
                        .getCurrentPosition(posOptions)
                        .then(function (position) {
                          //$log.info('getCurrentPosition then');
                          LocationData.coords.latitude = position.coords.latitude;
                          LocationData.coords.longitude = position.coords.longitude;

                          LocationData.isConsistent =true;

                          _this.getCurrentPlaceName(LocationData.coords, true);

                          $rootScope.$broadcast('mt-position-changed', position.coords);
                        }, function(err) {
                          LocationData.isConsistent = false;
                          $log.error('Impossibile rilevare posizione:'+ (err));
                        });
                    });
                },

                getCurrentPositionNoUpdate:function ( f ) {
                  var _this=this;
                    //$log.info('getCurrentPosition');
                    var posOptions = {timeout: 10000, enableHighAccuracy: false};
                    return $cordovaGeolocation.getCurrentPosition(posOptions);
                    // $ionicPlatform.ready(function () {
                      // will execute when device is ready, or immediately if the device is already ready.

                      // return $cordovaGeolocation.getCurrentPosition(posOptions);
                        // .then(function (position) {
                        //   //$log.info('getCurrentPosition then');
                        //
                        //
                        //   _this.getCurrentPlaceName(position.coords, false);
                        //
                        //   $rootScope.$broadcast('mt-position-changed', position.coords);
                        // }, function(err) {
                        //   LocationData.isConsistent = false;
                        //   $log.error('Impossibile rilevare posizione:'+ (err));
                        // });
                    // });
                },
                getLastPosition:function ( ) {
                    return LocationData.isConsistent ? LocationData.coords : false;
                },
                getLastPlaceName:function ( ) {
                    return LocationData.placeName;
                },
                getLocationData:function ( ) {
                    return LocationData;
                },
                setLocationData:function (placeName, lat, lon) {
                  $timeout(function () {
                    LocationData.placeName=placeName;
                    LocationData.coords.latitude=lat;
                    LocationData.coords.longitude=lon;
                      $rootScope.$broadcast('mt-location-data-changed', LocationData);
                  }, 100);


                },
                getTickedDecoded:function ( f ) {
                  if(!f || !f.tickets) return;

                  var retval = '';
                  var valuta = (f.invoice_profile) ? f.invoice_profile.currency :'';
                  var minStr='',maxStr='';
                  var min_price=0,max_price=0;

                  var min=-1,max=-1;

                  for (var i = 0; i < f.tickets.length; i++) {
                    if(f.tickets[i].active){
                      if(f.tickets[i].type=='free'){
                        min=-i;
                        minStr = 'FREE';
                      }else {//if(f.tickets[i].type=='donation')

                        if(min==-1 || f.tickets[i].base_price<min_price){
                          min = i;
                          min_price = f.tickets[i].base_price;
                          // minStr = valuta+' '+f.tickets[i].base_price.toFixed(2);
                        }
                        if(min!=i  && (max==-1 || f.tickets[i].base_price>max_price)){
                          max=i;
                          max_price = f.tickets[i].base_price;
                        }
                      }
                    }

                  }
                  if(min!=-1 && minStr!='FREE'){
                    minStr = valuta+' '+f.tickets[min].base_price.toFixed(2);
                  }
                  if(max!=-1){
                    maxStr = valuta+' '+f.tickets[max].base_price.toFixed(2);
                  }
                  return  minStr+(minStr!='' && maxStr!='' ? '/' : '') +maxStr;
                },
                getMonday:function ( date ) {
                    var day = date.getDay() || 7;
                    if( day !== 1 )
                        date.setHours(-24 * (day - 1));
                    return date;
                },

                getQRCode64: function (order_id, attendee_id) {

                    var url = ENV.apiUrl+'/order/'+order_id+'/attendee/'+attendee_id+'/qrcode';
                    var headers = {};
                    try {
                      if( AuthServices.getLoggedUser().access_token.token){
                        headers= {   "Authorization": "Bearer " + AuthServices.getLoggedUser().access_token.token }
                      }
                    } catch (e) { }

                    var req =  {
                       method: 'GET',
                       url: url,
                       headers: headers,
                        responseType:'arraybuffer',
                    }


                    var p = $q.defer();
                    $http(req).
                      then(function (response) {
                        var blob = new Blob(
                          [ response.data ],
                          { type: response.headers('Content-Type') }
                        );
                        var dataURL = ""; //="data:image/png;base64,"+encoded;

                        var reader = new window.FileReader();
                        reader.readAsDataURL(blob);
                        reader.onloadend = function() {
                            dataURL = unescape(reader.result);
                            p.resolve({
                              state: 'OK',
                              qrcode64: dataURL,//$base64.encode(response.data),
                            });
                            //console.log((dataURL );
                        }

                    }, function (response) {
                        var responseMsg = response.data && response.data.msg ?
                            response.data.msg :
                            "Errore remoto. Non riesco a comunicare con il server!";
                        p.reject({ state: 'KO', msg: responseMsg });
                    });
                    return p.promise;
                },
                sendMessage: function (id_profile, msg) {

                    // var msg = {  flyer: '560d49d70ff57d7b1bc4a769',
                    // name: 'nome',
                    // email: 'salvioma@gmail.com',
                    // body: 'Messaggio',
                    // ip: '' };
                    //
                    var headers = {};
                    try {
                        headers= ( AuthServices.getLoggedUser().access_token.token)
                          ? { "Authorization": "Bearer " + AuthServices.getLoggedUser().access_token.token }
                          : {};
                    } catch (e) { }

                    var req =  {
                       method: 'POST',
                       url: ENV.apiUrl+'user/'+id_profile+'/messages/' ,
                       headers: headers,
                       data: msg
                    }

                    return $http(req).
                      then(
                          function (response) {
                              return {
                                  state: 'OK',
                                  data: response.data,
                              };
                          },
                          function (response) {
                              var responseMsg = response.data && response.data.msg
                                  ? response.data.msg
                                  : "Errore remoto. Non riesco a comunicare con il server!";

                              var st = (response.data.error && response.data.error=='elapsed_time_error') ? 'KOT' :'KO';

                              return {
                                  state: st,
                                  msg: responseMsg
                              };
                          }
                      );
                },
                checkDiscountCode: function (_id, code) {
                    var headers = {};
                    try {
                        headers= ( AuthServices.getLoggedUser().access_token.token)
                          ? { "Authorization": "Bearer " + AuthServices.getLoggedUser().access_token.token }
                          : {};
                    } catch (e) { }

                    var req =  {
                       method: 'GET',
                       url: ENV.apiUrl+'/flyer/'+_id+'/discount_code_check/'+code ,
                       headers: headers,
                    }

                    return $http(req).
                      then(
                          function (response) {
                              return {
                                  state: 'OK',
                                  data: response.data,
                              };
                          },
                          function (response) {
                              var responseMsg = response.data && response.data.msg
                                  ? response.data.msg
                                  : "Errore remoto. Non riesco a comunicare con il server!";

                              return {
                                  state: 'KO',
                                  msg: responseMsg
                              };
                          }
                      );
                },
                checkPayPalPayment: function (pay_key) {
                    var headers = {};
                    try {
                        headers= ( AuthServices.getLoggedUser().access_token.token)
                          ? { "Authorization": "Bearer " + AuthServices.getLoggedUser().access_token.token }
                          : {};
                    } catch (e) { }

                    var req =  {
                       method: 'GET',
                //       url: ENV.apiUrl+'/sbx/paypal/check/'+encodeURIComponent(pay_key) ,
                       url: ENV.apiUrl+'paypal/check/'+encodeURIComponent(pay_key) ,
                       headers: headers,
                    }

                    return $http(req).
                      then(
                          function (response) {
                              return {
                                  state: 'OK',
                                  data: response.data,
                              };
                          },
                          function (response) {
                              var responseMsg = response.data && response.data.msg
                                  ? response.data.msg
                                  : "Errore remoto. Non riesco a comunicare con il server!";

                              return {
                                  state: 'KO',
                                  msg: responseMsg
                              };
                          }
                      );
                },
                deleteOrder: function (trnx_id) {
                    var headers = {};
                    try {
                        headers= ( AuthServices.getLoggedUser().access_token.token)
                          ? { "Authorization": "Bearer " + AuthServices.getLoggedUser().access_token.token }
                          : {};
                    } catch (e) { }

                    var req =  {
                       method: 'DELETE',
                       url: ENV.apiUrl+'/orders/'+trnx_id ,
                       headers: headers,
                    }

                    return $http(req).
                      then(
                          function (response) {
                              return {
                                  state: 'OK',
                                  data: response.data,
                              };
                          },
                          function (response) {
                              var responseMsg = response.data && response.data.msg
                                  ? response.data.msg
                                  : "Errore remoto. Non riesco a comunicare con il server!";

                              return {
                                  state: 'KO',
                                  msg: responseMsg
                              };
                          }
                      );
                },
                postFinalizeOrders: function (data) {
                    var headers = {};
                    try {
                        headers= ( AuthServices.getLoggedUser().access_token.token)
                          ? { "Authorization": "Bearer " + AuthServices.getLoggedUser().access_token.token }
                          : {};
                    } catch (e) { }

                    var req =  {
                       method: 'PUT',
                  //     url: ENV.apiUrl+'sbx/flyer/'+data.id_event+'/order/'+data.id_trnx ,
                       url: ENV.apiUrl+'flyer/'+data.id_event+'/order/'+data.id_trnx ,
                       headers: headers,
                       data: data.data
                    }

                    return $http(req).
                      then(
                          function (response) {
                              return {
                                  state: 'OK',
                                  data: response.data,
                              };
                          },
                          function (response) {
                              var responseMsg = response.data && response.data.msg
                                  ? response.data.msg
                                  : "Errore remoto. Non riesco a comunicare con il server!";

                              return {
                                  state: 'KO',
                                  msg: responseMsg
                              };
                          }
                      );
                },
                postBlockTickets: function (data) {
                    var headers = {};
                    try {
                        headers= ( AuthServices.getLoggedUser().access_token.token)
                          ? { "Authorization": "Bearer " + AuthServices.getLoggedUser().access_token.token }
                          : {};
                    } catch (e) { }

                    var req =  {
                       method: 'POST',
                      // url: ENV.apiUrl+'sbx/flyer/'+data.id_event+'/orders ' ,
                       url: ENV.apiUrl+'flyer/'+data.id_event+'/orders ' ,
                       headers: headers,
                       data: data.tickets
                    }

                    return $http(req).
                      then(
                          function (response) {
                              return {
                                  state: 'OK',
                                  data: response.data,
                              };
                          },
                          function (response) {
                              var responseMsg = response.data && response.data.msg
                                  ? response.data.msg
                                  : "Errore remoto. Non riesco a comunicare con il server!";

                              return {
                                  state: 'KO',
                                  msg: responseMsg
                              };
                          }
                      );
                },
                getUserTickets: function (page) {
                    var headers = {};
                    try {
                      if( AuthServices.getLoggedUser().access_token.token){
                        headers= {
                            "Authorization": "Bearer " + AuthServices.getLoggedUser().access_token.token
                        }
                      }
                    } catch (e) { }

                    var req =  {
                       method: 'GET',
                       url: ENV.apiUrl+'me/orders?page='+page ,
                       headers: headers,
                    }

                    return $http(req).
                      then(function (response) {

                        return {
                            state: 'OK',
                            data: response.data.data,
                            pagination: response.data.pagination
                        };

                    }, function (response) {

                        var responseMsg = response.data && response.data.msg ?
                            response.data.msg :
                            "Errore remoto. Non riesco a comunicare con il server!";

                        return {
                            state: 'KO',
                            msg: responseMsg
                        };
                    });
                },
                getUserEvents: function (filter) {

                  //[/user/{id}/public_events{?page}{?count}{?filter}
                    var headers = {};
                    try {
                      if( AuthServices.getLoggedUser().access_token.token){
                        headers= {
                            "Authorization": "Bearer " + AuthServices.getLoggedUser().access_token.token
                        }
                      }
                    } catch (e) { }

                    var req =  {
                       method: 'GET',
                       url: ENV.apiUrl+'user/'+ filter.id +'/public_events',
                       data:filter,
                       headers: headers,
                    }

                    return $http(req).
                      then(function (response) {

                        return {
                            state: 'OK',
                            flyers: response.data.flyers || [] ,
                            pagination: response.data.pagination || {},
                            is_private: (!response.data.flyers ? true : false)
                        };

                    }, function (response) {

                        var responseMsg = response.data && response.data.msg ?
                            response.data.msg :
                            "Errore remoto. Non riesco a comunicare con il server!";

                        return {
                            state: 'KO',
                            msg: responseMsg
                        };
                    });
                },
                getFlyers: function (page, filter, featured) {
                    var headers = {};
                    try {
                      if( AuthServices.getLoggedUser().access_token.token){
                        headers= {
                            "Authorization": "Bearer " + AuthServices.getLoggedUser().access_token.token
                        }
                      }
                    } catch (e) { }

                    var req =  {
                       method: featured ? 'GET' : 'POST',
                       url: featured ? ENV.apiUrl+'featured/client/1' :ENV.apiUrl+'search/flyers',
                       headers: headers,
                       data: filter
                    }

                    return $http(req).
                      then(function (response) {
                          if(featured){
                            var list = response.data;
                            var flyers = [];
                            for (var i = 0; i < list.length; i++) {
                              flyers.push(list[i].flyer);
                            }
                            return {
                                state: 'OK',
                                flyers: flyers,
                                pagination: {}
                            };
                          }
                          return {
                              state: 'OK',
                              flyers: response.data.flyers,
                              pagination: response.data.flyers_pagination
                          };

                      }, function (response) {

                          var responseMsg = response.data && response.data.msg ?
                              response.data.msg :
                              "Errore remoto. Non riesco a comunicare con il server!";

                          return {
                              state: 'KO',
                              msg: responseMsg
                          };
                      });
                  },

                  getFlyerDetails: function (flyerId) {
                      var headers = {};
                      try {
                        if( AuthServices.getLoggedUser().access_token.token){
                          headers= {
                              "Authorization": "Bearer " + AuthServices.getLoggedUser().access_token.token
                          }
                        }
                      } catch (e) { }

                      return $http.get(ENV.apiUrl + 'flyer/' + flyerId, {
                          headers: headers
                      }).
                      then(function (response) {
                          return {
                              state: 'OK',
                              details: response.data,
                          };

                      }, function (response) {

                          var responseMsg = response.data && response.data.msg ?
                              response.data.msg :
                              "Errore remoto. Non riesco a comunicare con il server.";

                          return {
                              state: 'KO',
                              msg: responseMsg
                          };

                      });
                  },

              };
          }

      ]);



    /* @ngInject */

})();
